/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.taglib;

import static org.opengion.fukurou.util.StringUtil.nval;

import java.util.Locale;
import java.util.Map;
import java.util.HashMap;
import java.util.Enumeration;

import org.opengion.fukurou.util.ErrorMessage;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.mail.MailManager_DIRECT;
import org.opengion.hayabusa.db.DBTableModel;

import javax.servlet.ServletRequest ;

/**
 * 定型文およびパラメータの設定によるメールを送信するためのタグです。
 *
 * @og.formSample
 * ●形式：&lt;og:mailSender ptnId="…" action="…" from="…" to="…" /&gt;
 * ●body：なし
 *
 * ●使用例
 *     &lt;og:mailSender2 &gt;
 *        ptnId     = PtnId      定型文ID（定型文マスタに登録されている定型文ID）
 *        action    = Action     アクション（CHECK：確認あり、SEND：確認後の送信、NOCHECK：確認なし）
 *        from      = From       送信元（送信者社員ID）
 *        to        = To         送信先（コンマ区切りで複数セット可能、社員ID、グループID）
 *        cc        = Cc         送信先（コンマ区切りで複数セット可能、社員ID、グループID）
 *        bcc       = Bcc        送信先（コンマ区切りで複数セット可能、社員ID、グループID）
 *        fileURL   = 添付ファイルのセーブディレクトリ
 *        filename  = 添付ファイル名（ローカルにセーブされたファイル名）（コンマ区切りで複数登録可能）
 *        addrCheck = true/false（メールアカウントの有効チェック）
 *        useStop   = true/false　エラー発生時に後続JSPの評価を中止する（true）/中止しない(false)
 *        scope     = request/session 宛先テーブルの格納スコープ（デフォルト：session）
 *        tableId   = TableId    宛先テーブルのID（通常はデフォルトのテーブルモデルID名称を利用します）
 *        debug     = true/false
 *     &lt;/og:mailSender &gt;
 *
 * from には社員IDしかセットできません。
 * to,cc,bccには社員ID、またはグループIDをコンマ区切りで複数セットできます。
 * action:CHECK は送信前に、一度送信内容を確認したい場合に利用します。action=CHECKの場合、scopeにはsessionしかセットできません。
 * action:SEND は確認済のメール文を送信する場合に利用します。
 * action:NOCHECK は確認なしで送信したい場合に利用します。
 *
 * @og.group その他出力
 *
 * @version  4.0
 * @author   Sen.Li
 * @since    JDK1.6
 */
public class MailSenderTag2 extends CommonTagSupport {
	private static final String		VERSION				= "4.4.0.0 (2009/01/05)";
	private static final long		serialVersionUID	= 4400;
	private static final String 	ACT_CHECK			= "CHECK" ;
	private static final String 	ACT_SEND			= "SEND" ;
	private static final String 	ACT_NOCHECK			= "NOCHECK" ;
	private static final int 		MAX_FILE_COUNT		= 5 ;
	private static final String[] 	ACTION_LIST = new String[] { ACT_CHECK , ACT_SEND, ACT_NOCHECK };
	private String	   ptnId  	 = null;
	private String	   action    = null;
	private String     from      = null;
	private String	   to        = null;
	private String	   cc        = null;
	private String	   bcc       = null;
	private String     fileURL   = HybsSystem.sys( "FILE_URL" );
	private String[]   filename  = null;
	private String	   tableId	 = HybsSystem.TBL_MDL_KEY ;
	private boolean	   addrCheck = false;
	private boolean	   useStop   = true;
	
	/**
	* Taglibの終了タグが見つかったときに処理する doEndTag() を オーバーライドします。
	*
	 * @return  int 後続処理の指示
	*/
	@Override
	public int doEndTag() {
		debugPrint();
		int rtnCode = EVAL_PAGE;
		int	errCode	= ErrorMessage.OK;

		if( check( action, ACTION_LIST ) ) {
			try {
				tableId = ( tableId == null ) ? HybsSystem.TBL_MDL_KEY:tableId;
				MailManager_DIRECT manager = new MailManager_DIRECT();
				manager.setResourceManager( getResource() );
				DBTableModel table = null;
				
				if ( ACT_NOCHECK.equals( action ) || ACT_CHECK.equals( action ) ){
					Map<String,String> initParamMap = makeParamMap();
					manager.create( initParamMap );
				}
				if( ACT_NOCHECK.equals( action ) ) {
					manager.setDebug( isDebug() );
					manager.send();
				}
				else if ( ACT_CHECK.equals( action ) ) {
					setSessionAttribute( "MAIL.FROM_ADDR", manager.getFromAddr() );
					setSessionAttribute( "MAIL.PTN_ID", ptnId );
					setSessionAttribute( "MAIL.TITLE", manager.getTitle() );
					setSessionAttribute( "MAIL.CONTENT", manager.getContent() );
				}
				else if ( ACT_SEND.equals( action ) ) {
					ptnId = (String) getSessionAttribute( "MAIL.PTN_ID" );
					Map<String,String> initParamMap = makeParamMap();
					manager.setFromAddr( (String) getSessionAttribute( "MAIL.FROM_ADDR" ) );
					manager.setTitle( (String) getSessionAttribute( "MAIL.TITLE" ) );
					manager.setContent( (String) getSessionAttribute( "MAIL.CONTENT" ) );
					table = ( DBTableModel )getObject( tableId );
					manager.create( initParamMap, table );
					manager.setDebug( isDebug() );
					manager.send();
				}
				startQueryTransaction( tableId );
				table = manager.makeDstTable();
				if( ! commitTableObject( tableId, table ) ) {
					jspPrint( "DBTableModel は登録しません。" );
				}
			}
			catch( RuntimeException rex ){
				if ( useStop ) {
					ErrorMessage errMsg = new ErrorMessage();
					errMsg.addMessage( 0, ErrorMessage.NG, "ERR0040", rex.getMessage() );
					jspPrint( TaglibUtil.makeHTMLErrorTable( errMsg, getResource() ) );
					rtnCode = SKIP_PAGE;
				}
				rex.printStackTrace();
				errCode = ErrorMessage.WARNING;
			}
			setSessionAttribute( "MAIL.ERR_CODE", String.valueOf( errCode ) );
		}
		else {
			String errMsg = "設定のアクションはサポートされていません：" + action ;
			throw new HybsSystemException( errMsg );
		}
		return rtnCode;
	}

	/**
	 * 【TAG】アクション(CHECK,SEND,NOCHECK)をセットします。
	 * @og.tag
	 * 送信前に、一度送信内容を確認する場合、"CHECK "をセットします。
	 * 確認済のメール文を送信する場合、"SEND"をセットします。
	 * 確認なしで送信する場合、"NOCHECK"をセットします。
	 * 
	 * @param	act String
	 */
	public void setAction( final String act ) {
		String act2 = getRequestParameter( act );
		if( act2 != null && act2.length() > 0 ) { action = act2.toUpperCase(Locale.JAPAN); }
	}

	/**
	 * 【TAG】メール定型文のIDを指定します。
	 *
	 * @og.tag
	 * 定型文マスタに定義されている定型文IDを指定します。
	 *
	 * @param   pid String
	 */
	public void setPtnId( final String pid ) {
		ptnId = nval( getRequestParameter( pid ),null );
	}

	/**
	 * 【TAG】送信元（ＦＲＯＭ）の社員IDを指定します。
	 *
	 * @og.tag 
	 * 送信元（ＦＲＯＭ）の社員IDを指定します。社員マスタに存在している社員ID(例："C12345")しかセットできません。
	 *
	 * @param   fromId String
	 */
	public void setFrom( final String fromId ) {
		from = nval( getRequestParameter( fromId ), from );
		setRequestAttribute( "FROM", from );
	}

	/**
	 * 【TAG】送信先（ＴＯ）の社員ID、グループIDをCSV形式で指定します。
	 *
	 * @og.tag
	 * 複数のID（社員ID、グループID）をカンマ区切りでセットできます。
	 * グループIDはグループマスタ管理画面により定義する必要があります。"GP.XXXXX"の形式でセットします。
	 *
	 * @param   toIds String
	 */
	public void setTo( final String toIds ) {
		to = getRequestParameter( toIds );
	}

	/**
	 * 【TAG】送信先（ＣＣ）の社員ID、グループIDをCSV形式で指定します。
	 *
	 * @og.tag
	 * 複数のID（社員ID、グループID）をカンマ区切りでセットできます。
	 * グループIDはグループマスタ管理画面により定義する必要があります。"GP.XXXXX"の形式でセットします。
	 *
	 * @param   ccIds String
	 */
	public void setCc( final String ccIds ) {
		cc = getRequestParameter( ccIds );
	}

	/**
	 * 【TAG】送信先（ＢＣＣ）の社員ID、グループIDをCSV形式で指定します。
	 *
	 * @og.tag
	 * 複数のID（社員ID、グループID）をカンマ区切りでセットできます。
	 * グループIDはグループマスタ管理画面により定義する必要があります。"GP.XXXXX"の形式でセットします。
	 *
	 * @param   bccIds String
	 */
	public void setBcc( final String bccIds ) {
		bcc = getRequestParameter( bccIds );
	}

	/**
	 * 【TAG】添付ファイルのセーブディレクトリを指定します(初期値:FILE_URL)。
	 *
	 * @og.tag
	 * この属性で指定されるディレクトリに、添付ファイルが存在すると仮定します。
	 * 指定方法は、通常の fileURL 属性と同様に、先頭が、'/' （UNIX) または、２文字目が、
	 * ":" （Windows）の場合は、指定のURLそのままのディレクトリに、そうでない場合は、
	 * システムパラメータ の FILE_URL 属性で指定のフォルダの下に、作成されます。
	 * fileURL = "{&#064;USER.ID}" と指定すると、FILE_URL 属性で指定のフォルダの下に、
	 * さらに、各個人ID別のフォルダを作成して、そこを使用します。
	 *
	 * @param	url 添付ファイルのセーブディレクトリ
	 */
	public void setFileURL( final String url ) {
		String furl = nval( getRequestParameter( url ),null );
		if( furl != null ) {
			char ch = furl.charAt( furl.length()-1 );
			if( ch != '/' && ch != '\\' ) { furl = furl + "/"; }
			fileURL = StringUtil.urlAppend( fileURL,furl );
		}
	}

	/**
	 * 【TAG】添付ファイル名を指定します。
	 *
	 * @og.tag
	 * 複数ファイルをセットできます。
	 * 設定方法は、カンマで区切って並べ複数指定できます。
	 *
	 * @param   fname String
	 */
	public void setFilename( final String fname ) {
		filename = StringUtil.csv2ArrayOnly( getRequestParameter( fname ) );
	}

	/**
	 * 【TAG】メールアドレスの構文とメールアカウントのチェックをする（true）/しない（false）を指定します。
	 *
	 * @og.tag
	 * メールアドレスの構文とメールアカウントのチェックをする（true）/しない（false）を指定します。
	 * メール文合成の段階では、メールアドレスの構文文法についてチェックします。
	 * メール送信の段階では、メールアカウントが有効かについてチェックします。
	 * "true"と指定する場合、ｴﾗｰが検出されたら、例外を投げて本タグの処理が中止されます。
	 * "false"と指定する場合、ｴﾗｰが検出されても、例外を投げません。
	 *
	 * @param   addrChk String
	 */
	public void setAddrCheck( final String addrChk ) {
		addrCheck = nval( getRequestParameter( addrChk ), addrCheck );
	}

	/**
	 * 【TAG】例外発生した場合、後続JSPの評価を中止する（true）/しない(false)かを指定します。
	 *
	 * @og.tag
	 * "true"と指定する場合、例外が発生したら、後続JSPが評価されません。
	 * "false"と指定する場合、例外が発生しても、後続JSPが評価されます。後続のJSPでは変数
	 * {@MAIL.ERR_CODE}で本タグの実行状況（ｴﾗｰ発生したか）を取得できます。
	 *
	 * @param   stop String
	 */
	public void setUseStop( final String stop ) {
		useStop = nval( getRequestParameter( stop ), useStop );
	}

	/**
	 * 【TAG】(通常は使いません)宛先のDBTableModelを、sessionに登録するときのキーを指定します。
	 *
	 * @og.tag
	 * 本タグでは、宛先のリストに基づいて DBTableModel オブジェクトを作成します。
	 * この tableId は sessionに登録するときのキーとして指定します。
	 * 初期値は、HybsSystem.TBL_MDL_KEY です。
	 *
	 * @param	id String
	 */
	public void setTableId( final String id ) {
		tableId   = nval( getRequestParameter( id ),tableId );	// 3.8.0.9 (2005/10/17)
	}

	/**
	 * タグリブオブジェクトをリリースします。
	 * キャッシュされて再利用されるので、フィールドの初期設定を行います。
	 * 
	 */
	@Override
	protected void release2() {
		super.release2();
		from      = null;
		to        = null;
		cc        = null;
		bcc       = null;
		fileURL   = HybsSystem.sys( "FILE_URL" );
		filename  = null;
		ptnId  	  = null;
		action    = null;
		tableId	  = HybsSystem.TBL_MDL_KEY ;
		addrCheck = false;
		useStop   = true;
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return このクラスの文字列表現
	 */
	public String toString() {
		return org.opengion.fukurou.util.ToString.title(this.getClass().getName() )
		.println( "VERSION"		,VERSION 	)
		.println( "ptnId"		,ptnId		)
		.println( "action"		,action		)
		.println( "tableId"		,tableId	)
		.println( "addrCheck"	,addrCheck	)
		.println( "useStop"		,useStop	)
		.println( "from" 		,from 		)
		.println( "to"			,to			)
		.println( "cc"			,cc			)
		.println( "bcc"			,bcc		)
		.println( "filename" 	,filename 	)
		.println( "fileURL" 	,fileURL 	)
		.println( "Other...", getAttributes().getAttribute() )
		.fixForm().toString();
	}
	
	/**
	 * リクエスト変数の値より、定型文に必要なパラメータを取得して、パレメータマップに入れます。
	 * パラメータマップは引数としてメールモジュールのマネージャに渡します。
	 * マネージャの中には、定型文を元に、パラメータマップの値とマージしてメールの各項目を合成します。
	 * 
	 * @return paramMap Map<string,String>
	 */
	private Map<String, String> makeParamMap() {
		Map<String, String> paramMap = new HashMap<String, String>();
		if( action.endsWith( ACT_NOCHECK ) || action.equals( ACT_CHECK ) ) {
			ServletRequest request = this.getRequest();
			Enumeration<?> enu1 = request.getAttributeNames();
			while( enu1.hasMoreElements() ) {
				String name = (String) enu1.nextElement();
				Object tmpObj = request.getAttribute( name );
				if ( tmpObj instanceof String ) {
					paramMap.put( name, (String) request.getAttribute( name ) );
				}
			}
			Enumeration<?> enu2 = request.getParameterNames();
			while( enu2.hasMoreElements() ) {
				String name = (String) enu2.nextElement();
				paramMap.put( name, request.getParameter( name ) );
			}

			paramMap.put( "FROM", from );
			paramMap.put( "TO", to );
			paramMap.put( "CC", cc );
			paramMap.put( "BCC", bcc );
		}
		paramMap.put( "PTN_ID", ptnId );
		paramMap.put( "SYSTEM_ID", HybsSystem.sys( "SYSTEM_ID" ) );
		paramMap.put( "ADDR_CHECK", String.valueOf( addrCheck ) );
		paramMap.put( "LOGIN_USERID", getRequestValue( "USER.ID" ) );
		paramMap.put( "LOGIN_USERNAME", getRequestValue( "USER.JNAME" ) );
		paramMap.put( "PGID", getRequestValue( "GUI.KEY" ) );
		paramMap.put( "DATE", HybsSystem.getDate( "yyyy/MM/dd" ) );
		paramMap.put( "TIME", HybsSystem.getDate( "HH:mm:ss" ) );

		String[] temp = { "", "", "", "", "" };
		if( filename != null && filename.length > 0 ) {
			String directory = HybsSystem.url2dir( fileURL );
			int fileCount = filename.length > MAX_FILE_COUNT ? MAX_FILE_COUNT : filename.length;
			for( int i = 0; i < fileCount; i++ ) {
				temp[i] = StringUtil.urlAppend( directory, filename[i] );
			}
		}
		paramMap.put( "ATTACH1", temp[0] );
		paramMap.put( "ATTACH2", temp[1] );
		paramMap.put( "ATTACH3", temp[2] );
		paramMap.put( "ATTACH4", temp[3] );
		paramMap.put( "ATTACH5", temp[4] );
		return paramMap;
	}
}
